package xyz.lychee.lagfixer.libs.adventure.util;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.EnumSet;
import java.util.Iterator;
import java.util.List;
import java.util.Objects;
import java.util.Set;
import java.util.function.Function;
import xyz.lychee.lagfixer.libs.annotations.NotNull;

public final class MonkeyBars {
   private MonkeyBars() {
   }

   @SafeVarargs
   public static <E extends Enum<E>> @NotNull Set<E> enumSet(final Class<E> type, final @NotNull E... constants) {
      Set<E> set = EnumSet.noneOf(type);
      Collections.addAll(set, constants);
      return Collections.unmodifiableSet(set);
   }

   public static <T> @NotNull List<T> addOne(final @NotNull List<T> oldList, final T newElement) {
      if (oldList.isEmpty()) {
         return Collections.singletonList(newElement);
      } else {
         List<T> newList = new ArrayList(oldList.size() + 1);
         newList.addAll(oldList);
         newList.add(newElement);
         return Collections.unmodifiableList(newList);
      }
   }

   @SafeVarargs
   public static <I, O> @NotNull List<O> nonEmptyArrayToList(final @NotNull Function<I, O> mapper, final @NotNull I first, final @NotNull I... others) {
      List<O> ret = new ArrayList(others.length + 1);
      ret.add(mapper.apply(first));
      Object[] var4 = others;
      int var5 = others.length;

      for(int var6 = 0; var6 < var5; ++var6) {
         I other = var4[var6];
         ret.add(Objects.requireNonNull(mapper.apply(Objects.requireNonNull(other, "source[?]")), "mapper(source[?])"));
      }

      return Collections.unmodifiableList(ret);
   }

   public static <I, O> @NotNull List<O> toUnmodifiableList(final @NotNull Function<I, O> mapper, final @NotNull Iterable<? extends I> source) {
      ArrayList<O> ret = source instanceof Collection ? new ArrayList(((Collection)source).size()) : new ArrayList();
      Iterator var3 = source.iterator();

      while(var3.hasNext()) {
         I el = var3.next();
         ret.add(Objects.requireNonNull(mapper.apply(Objects.requireNonNull(el, "source[?]")), "mapper(source[?])"));
      }

      return Collections.unmodifiableList(ret);
   }
}
